//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "wclIrDAClients"
#pragma link "wclIrDADevices"
#pragma resource "*.dfm"

#ifdef _WIN64
#pragma comment(lib, "wclCommon.a")
#pragma comment(lib, "wclCommunication.a")
#pragma comment(lib, "wclIrDAFrameworkR.a")
#else
#pragma comment(lib, "wclCommon.lib")
#pragma comment(lib, "wclCommunication.lib")
#pragma comment(lib, "wclIrDAFrameworkR.lib")
#endif

TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormCreate(TObject *Sender)
{
  // We use ObjectPush Profile so must set the profile's UUID.
  wclIrDAClient->Service = "OBEX";
  wclIrDAClient->Mode = cmIrComm3Wire;

  FOpp = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btClearClick(TObject *Sender)
{
  lbLog->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::FormDestroy(TObject *Sender)
{
  wclIrDAClient->Disconnect();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btDiscoverClick(TObject *Sender)
{
  lvDevices->Clear();
  
  // Discover classic devices only.
  TwclIrDADevices Devices;
  int Res = wclIrDAReceiver->Discover(Devices);
  if (Res != WCL_E_SUCCESS)
      MessageDlg("Failed to start discovering: 0x" + IntToHex(Res, 8),
        mtError, TMsgDlgButtons() << mbOK, 0);
  else
    for (int i = 0; i < Devices.Length; i++)
    {
      TListItem* Item = lvDevices->Items->Add();
      Item->Caption = IntToHex((int)Devices[i].Address, 8);
      Item->SubItems->Add(Devices[i].Name);
    }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btDisconnectClick(TObject *Sender)
{
  int Res = wclIrDAClient->Disconnect();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Disconnect failed: 0x" + IntToHex(Res, 8), mtError,
      TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btCloseSessionClick(TObject *Sender)
{
  if (FOpp == NULL)
    MessageDlg("Not connected", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    int Res = FOpp->Disconnect("");
    if (Res != WCL_E_SUCCESS)
      MessageDlg("Close session failed: 0x" + IntToHex(Res, 8), mtError,
        TMsgDlgButtons() << mbOK, 0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btOpenSessionClick(TObject *Sender)
{
  if (FOpp == NULL)
    MessageDlg("Not connected", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    int Res = FOpp->Connect();
    if (Res != WCL_E_SUCCESS)
      MessageDlg("Open session failed: 0x" + IntToHex(Res, 8), mtError,
        TMsgDlgButtons() << mbOK, 0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::OppConnect(TObject* Sender, const int Error,
  const String Description)
{
  lbLog->Items->Add("OBEX session opened with result: 0x" + IntToHex(Error, 8) +
    ". Description: " + Description);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::OppDisconnect(TObject* Sender, const int Error,
  const String Description)
{
  lbLog->Items->Add("OBEX session closed with result: 0x" + IntToHex(Error, 8) +
    ". Description: " + Description);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btConnectClick(TObject *Sender)
{
  if (wclIrDAClient->State != csDisconnected)
    MessageDlg("Client is connected", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
    if (lvDevices->Selected == NULL)
      MessageDlg("Select device", mtWarning, TMsgDlgButtons() << mbOK, 0);

    else
    {
      int Addr = StrToInt("$" + lvDevices->Selected->Caption);
      wclIrDAClient->Address = Addr;
      int Res = wclIrDAClient->Connect();
      if (Res != WCL_E_SUCCESS)
        MessageDlg("Failed to connect: 0x" + IntToHex(Res, 8),
          mtError, TMsgDlgButtons() << mbOK, 0);
    }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btSelectFileClick(TObject *Sender)
{
  if (OpenDialog->Execute())
    edFileName->Text = OpenDialog->FileName;
}
//---------------------------------------------------------------------------

#if __BORLANDC__ >= 0x0740
  void __fastcall TfmMain::OppPutComplete(TObject* Sender, const int Error,
	const String Description, TStream* const Stream)
#else
  void __fastcall TfmMain::OppPutComplete(TObject* Sender, const int Error,
	const String Description, const TStream* Stream)
#endif
{
  lbLog->Items->Add("File sending completed with result: 0x" +
    IntToHex(Error, 8) + ". Description: " + Description);

  ProgressBar->Position = 0;
  ProgressBar->Max = 0;

  // We must destroy the stream object here.
  ((TStream*)Stream)->Free();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::OppProgress(TObject* Sender, const unsigned Size,
  const unsigned Position)
{
  ProgressBar->Max = Size;
  ProgressBar->Position = Position;
}
//---------------------------------------------------------------------------


void __fastcall TfmMain::btAbortClick(TObject *Sender)
{
  if (FOpp == NULL)
    MessageDlg("Not connected", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    int Res = FOpp->Abort("");
    if (Res != WCL_E_SUCCESS)
      MessageDlg("Open session failed: 0x" + IntToHex(Res, 8), mtError,
        TMsgDlgButtons() << mbOK, 0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btSendFileClick(TObject *Sender)
{
  if (FOpp == NULL)
    MessageDlg("Not connected", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
    if (edFileName->Text == "")
      MessageDlg("Select file", mtWarning, TMsgDlgButtons() << mbOK, 0);
    else
    {
      TFileStream* Stream = new TFileStream(edFileName->Text, fmOpenRead);
      int Res = FOpp->Put(ExtractFileName(edFileName->Text), "", Stream);
      if (Res != WCL_E_SUCCESS)
      {
        MessageDlg("Send file failed: 0x" + IntToHex(Res, 8), mtError,
          TMsgDlgButtons() << mbOK, 0);
        Stream->Free();
      }
    }        
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btGetvCardClick(TObject *Sender)
{
  if (FOpp == NULL)
    MessageDlg("Not connected", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    TFileStream* Stream = new TFileStream("vcard.vcf", fmCreate);
    // Request default vCard object.
    int Res = FOpp->Get("text/x-vCard", Stream);
    if (Res != WCL_E_SUCCESS)
    {
      MessageDlg("GetvCard failed: 0x" + IntToHex(Res, 8), mtError,
        TMsgDlgButtons() << mbOK, 0);
      ((TStream*)Stream)->Free();
    }
  }        
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btGetCapsClick(TObject *Sender)
{
  if (FOpp == NULL)
    MessageDlg("Not connected", mtWarning, TMsgDlgButtons() << mbOK, 0);
  else
  {
    TFileStream* Stream = new TFileStream("caps.xml", fmCreate);
    // Request default vCard object.
    int Res = FOpp->Get("x-obex/capability", Stream);
    if (Res != WCL_E_SUCCESS)
    {
      MessageDlg("GetCaps failed: 0x" + IntToHex(Res, 8), mtError,
        TMsgDlgButtons() << mbOK, 0);
      Stream->Free();
    }
  }
}
//---------------------------------------------------------------------------

#if __BORLANDC__ >= 0x0740
  void __fastcall TfmMain::OppGetComplete(TObject* Sender, const int Error,
	const String Description, TStream* const Stream)
#else
  void __fastcall TfmMain::OppGetComplete(TObject* Sender, const int Error,
    const String Description, const TStream* Stream)
#endif
{
  lbLog->Items->Add("File receiving completed with result: 0x" +
    IntToHex(Error, 8) + ". Description: " + Description);

  ProgressBar->Position = 0;
  ProgressBar->Max = 0;

  // We must destroy the stream object here.
  ((TStream*)Stream)->Free();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAClientDisconnect(TObject *Sender,
      const int Reason)
{
  lbLog->Items->Add("Disconnected with reason: 0x" + IntToHex(Reason, 8));
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAClientConnect(TObject *Sender,
      const int Error)
{
  lbLog->Items->Add("Connect. Operation result: 0x" + IntToHex(Error, 8));        
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAClientCreateProcessor(TObject *Sender,
      const TwclClientDataConnection *Connection)
{
  // here we must create the data processor for the connection. In this demo
  // we use OPPClient.
  #if __BORLANDC__ >= 0x0650
	FOpp = new TwclObexOppClient((TwclClientDataConnection*)Connection);
	#if __BORLANDC__ >= 0x0740
	  FOpp->OnGetComplete = OppGetComplete;
	  FOpp->OnPutComplete = OppPutComplete;
	#else
	  FOpp->OnGetComplete = (TwclObexObjectOperationResultEvent)&OppGetComplete;
	  FOpp->OnPutComplete = (TwclObexObjectOperationResultEvent)&OppPutComplete;
	#endif
  #else
	FOpp = new TwclObexOppClient(Connection);
	FOpp->OnGetComplete = OppGetComplete;
	FOpp->OnPutComplete = OppPutComplete;
  #endif
  FOpp->OnConnect = OppConnect;
  FOpp->OnDisconnect = OppDisconnect;
  FOpp->OnProgress = OppProgress;
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclIrDAClientDestroyProcessor(TObject *Sender,
      const TwclClientDataConnection *Connection)
{
  // Do we have any data processor created?
  if (FOpp != NULL)
    // Make sure it is our connection is goind to destroy.
    if (Connection->Processor == FOpp)
    {
      // Ok, destroy the data processor here.
      FOpp->Free();
      FOpp = NULL;
    }        
}
//---------------------------------------------------------------------------

